@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009 Net-ng.
+# Copyright (c) 2008, 2009, 2010 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -61,19 +61,17 @@ def create_rules(app_names, error):
 
         static = aconf['application'].get('static', os.path.join(dist.location, 'static') if dist else None)
 
-        if static and not os.path.isdir(static):
-            static = None
-
-        apps.append((app_name, static))
+        if static and os.path.isdir(static):
+            apps.append((aconf['application']['name'], static))
 
     return sorted(apps, key=lambda x: len(x[0]))
 
 def commonprefix(filenames):
     """Return the common prefix of a list of filenames
-    
+
     In:
       - ``filenames`` -- the list of filenames
-      
+
     Return:
       - the common prefix
     """
@@ -129,7 +127,7 @@ def generate_apache_rules(app_names, error):
     """
     apps = create_rules(app_names, error)
     document_root = commonprefix([static for (name, static) in apps])
-    
+
     yield 'DocumentRoot %s' % document_root
     yield ''
 
