@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009 Net-ng.
+# Copyright (c) 2008, 2009, 2010 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -17,16 +17,14 @@ from nagare import presentation
 
 class Admin(object):
     priority = 100        # Order of the default view, into the administrative interface
-    
+
     def __init__(self, apps):
         """Initialization
-        
-        In;
-          - ``apps`` -- a dictionary where the keys are the application objects
-            and the keys a tuple (application name, application urls)
-        """
-        self.apps = sorted(apps.values(), key=operator.itemgetter(1))
 
+        In:
+          - ``apps`` -- list of tuples (application, application name, application urls)
+        """
+        self.apps = sorted([(app_name, names) for (_, app_name, names) in apps])
 
 @presentation.render_for(Admin)
 def render(self, h, *args):
@@ -34,6 +32,7 @@ def render(self, h, *args):
     """
     with h.div:
         h << h.h2('Active applications')
+
         with h.ul:
             for (app_name, names) in self.apps:
                 with h.li("Application '%s' registered as " % app_name):
