@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009 Net-ng.
+# Copyright (c) 2008, 2009, 2010 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -16,13 +16,12 @@ from nagare import component, presentation
 
 class Admin(object):
     priority = 1        # Order of the default view, into the administrative interface
-    
+
     def __init__(self, apps):
         """Initialisation
-        
+
         In:
-          - ``apps`` -- a dictionary where the keys are the application objects
-            and the keys a tuple (application name, application urls)
+          - ``apps`` -- list of tuples (application, application name, application urls)
         """
         pass
 
@@ -31,7 +30,7 @@ def render(self, h, *args):
     """Display informations about the running framework environment"""
     return h.div(
         h.h2('Informations'),
-        
+
         h.p('Nagare - version %s - %s -' % (pkg_resources.get_distribution('nagare').version, time.strftime('%c'))),
         # h.p('Installed options: ', ', '.join(pkg_resources.get_distribution('fw-ng').extras)),
 
