@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009 Net-ng.
+# Copyright (c) 2008, 2009, 2010 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -21,7 +21,7 @@ In both cases:
 
 import sys, os, code, atexit, __builtin__
 
-from nagare import database
+from nagare import database, log
 from nagare.admin import util
 
 def create_globals(cfgfiles, debug, error):
@@ -32,13 +32,32 @@ def create_globals(cfgfiles, debug, error):
       - ``debug`` -- enable the display of the generated SQL statements
       - ``error`` -- the function to call in case of configuration errors
     """
-    
-    apps = {}
-             
-    # For each application given, activate its metadata
+
+    configs = []
+
+    # Merge all the ``[logging]]`` section of all the applications
     for cfgfile in cfgfiles:
+        # Read the configuration file of the application
         (cfgfile, app, dist, aconf) = util.read_application(cfgfile, error)
-        apps[aconf['application']['name']] = util.activate_WSGIApp(app, cfgfile, aconf, error, debug=debug)[0]
+        configs.append((cfgfile, app, dist, aconf))
+
+        log.configure(aconf['logging'].dict(), aconf['application']['name'])
+
+    # Configure the logging system
+    log.activate()
+
+    apps = {}
+
+    # For each application given, activate its metadata and its logger
+    for (cfgfile, app, dist, aconf) in configs:
+        name = aconf['application']['name']
+
+        log.set_logger('nagare.application.'+name)
+
+        (apps[name], databases) = util.activate_WSGIApp(app, cfgfile, aconf, error, debug=debug)
+
+        for (database_settings, populate) in databases:
+            database.set_metadata(*database_settings)
 
     session = database.session
     session.begin()
@@ -170,9 +189,9 @@ def batch(parser, options, args):
     __builtin__.__dict__.update(ns)
 
     util.load_file(args[1], None)
-    
+
 class Batch(util.Command):
     desc = 'Execute Python statements from a file'
-    
+
     set_options = staticmethod(set_batch_options)
     run = staticmethod(batch)
