@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009 Net-ng.
+# Copyright (c) 2008, 2009, 2010 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -14,7 +14,7 @@ from nagare.namespaces import xhtml_base
 from nagare.namespaces import xml
 from nagare import callbacks
 from nagare import wsgi
-from nagare.sessions import threaded_sessions
+from nagare.sessions.memory_sessions import SessionsWithPickledStates
 from nagare import presentation
 
 from types import ListType
@@ -25,11 +25,11 @@ import os
 
 def create_FixtureApp(app):
     app = wsgi.create_WSGIApp(app)
-    app.start(threaded_sessions.SessionsFactory('', {}, lambda x: None)())
+    app.set_sessions_manager(SessionsWithPickledStates())
+    app.start()
 
     return TestApp(app)
 
-
 # Test for XHTML namespace
 
 def head_render_init_test1():
