@@ -28,10 +28,10 @@ class Klass:
         self.name = name
         self.klasses[name] = self
         self.functions = set()
-        
+
     def set_base(self, base_name):
         self.base = self.klasses.get(base_name)
-        
+
     def add_function(self, function_name):
         self.functions.add(function_name)
 
@@ -47,7 +47,7 @@ class TranslationError(Exception):
 class Translator:
 
     def __init__(self, module_name, mod, output):
-    
+
         if module_name:
             self.module_prefix = module_name + "_"
         else:
@@ -91,18 +91,18 @@ class Translator:
         function_name = self.module_prefix + node.name
         function_args = "(" + ", ".join(node.argnames) + ")"
         print >>self.output, "function %s%s {" % (function_name, function_args)
-    
+
         if node.doc:
             print >>self.output, node.doc
-        
+
         else:
             for child in node.code:
                 self._stmt(child, None)
-            
+
         print >>self.output, "}"
         print >>self.output, "\n"
-    
-    
+
+
     def _return(self, node, current_klass):
         expr = self.expr(node.value, current_klass)
         if expr != "null":
@@ -168,14 +168,14 @@ class Translator:
                 raise TranslationError("unsupported type (in _callfunc)", v.node.expr)
         else:
             raise TranslationError("unsupported type (in _callfunc)", v.node)
-    
+
         for ch4 in v.args:
             arg = self.expr(ch4, current_klass)
             call_args.append(arg)
 
         return call_name + "(" + ", ".join(call_args) + ")"
-    
-    
+
+
     def _getattr(self, v):
         attr_name = v.attrname
         if isinstance(v.expr, ast.Name):
@@ -197,8 +197,8 @@ class Translator:
             return self._callfunc(v.expr, self.module_prefix) + "." + attr_name
         else:
             raise TranslationError("unsupported type (in _getattr)", v.expr)
-    
-    
+
+
     def _name(self, v):
         if v.name == "True":
             return "true"
@@ -257,19 +257,19 @@ class Translator:
             call_name = self._getattr2(v.expr, current_klass, v.attrname + "." + attr_name)
         else:
             raise TranslationError("unsupport type (in _getattr2)", v.expr)
-            
+
         return call_name
-    
-    
+
+
     def _class(self, node):
-    
+
         class_name = self.module_prefix + node.name
         current_klass = Klass(node.name)
-        
+
         for child in node.code:
             if isinstance(child, ast.Function):
                 current_klass.add_function(child.name)
-        
+
         if len(node.bases) == 0:
             base_class = ""
         elif len(node.bases) == 1:
@@ -287,7 +287,7 @@ class Translator:
             current_klass.set_base(base_class)
         else:
             raise TranslationError("more than one base (in _class)", node)
-        
+
         print >>self.output, "__" + class_name + '.prototype.__class__ = "' + class_name + '";'
 
         if "__init__" not in current_klass.functions:
@@ -299,7 +299,7 @@ class Translator:
             if base_class:
                 print >>self.output, "    __" + base_class + ".call(this);"
             print >>self.output, "}"
-    
+
         for child in node.code:
             if isinstance(child, ast.Pass):
                 pass
@@ -309,14 +309,14 @@ class Translator:
                 self.classattr(child, current_klass)
             else:
                 raise TranslationError("unsupported type (in _class)", child)
-    
+
     def classattr(self, node, current_klass):
         self._assign(node, current_klass, True)
-    
+
     def _method(self, node, current_klass, class_name):
         # reset global var scope
         self.method_imported_globals = set()
-        
+
         arg_names = node.argnames
 
         if len(arg_names) == 0:
@@ -324,7 +324,7 @@ class Translator:
         if arg_names[0] != "self":
             raise TranslationError("first arg not 'self' (in _method)", node)
         function_args = "(" + ", ".join(arg_names[1:]) + ")"
-    
+
         if node.name == "__init__":
             print >>self.output, "function " + class_name + function_args + " {"
             print >>self.output, "    return new __" + class_name + function_args + ";"
@@ -343,18 +343,18 @@ class Translator:
                     default_value = self._name(default_node)
                 else:
                     raise TranslationError("unsupported type (in _method)", default_node)
-                
+
                 default_name = arg_names[default_pos]
                 default_pos += 1
                 print >>self.output, "    if (typeof %s == 'undefined') %s=%s;" % (default_name, default_name, default_value)
 
         if node.doc:
             print >>self.output, node.doc
-        
+
         else:
             for child in node.code:
                 self._stmt(child, current_klass)
-            
+
         if node.name == "__init__":
             print >>self.output, "}"
         else:
@@ -388,8 +388,8 @@ class Translator:
             pass
         else:
             raise TranslationError("unsupported type (in _stmt)", node)
-    
-    
+
+
     def _augassign(self, node, current_klass):
         v = node.node
         if isinstance(v, ast.Getattr):
@@ -400,7 +400,7 @@ class Translator:
         rhs = self.expr(node.expr, current_klass)
         print >>self.output, "    " + lhs + " " + op + " " + rhs + ";"
 
-    
+
     def _assign(self, node, current_klass, top_level = False):
         if len(node.nodes) != 1:
             raise TranslationError("not single node (in _assign)", node)
@@ -431,7 +431,7 @@ class Translator:
                 op = "="
             else:
                 raise TranslationError("unsupported flag (in _assign)", v)
-    
+
         elif isinstance(v, ast.AssName):
             if top_level:
                 if current_klass:
@@ -462,12 +462,12 @@ class Translator:
                 raise TranslationError("unsupported flag (in _assign)", v)
         else:
             raise TranslationError("unsupported type (in _assign)", v)
-    
+
 
         rhs = self.expr(node.expr, current_klass)
         print >>self.output, lhs + " " + op + " " + rhs + ";"
-    
-    
+
+
     def _discard(self, node, current_klass):
         if isinstance(node.expr, ast.CallFunc):
             expr = self._callfunc(node.expr, current_klass)
@@ -476,8 +476,8 @@ class Translator:
             print >>self.output, node.expr.value
         else:
             raise TranslationError("unsupported type (in _discard)", node.expr)
-    
-    
+
+
     def _if(self, node, current_klass):
         for i in range(len(node.tests)):
             test, consequence = node.tests[i]
@@ -487,20 +487,20 @@ class Translator:
                 keyword = "else if"
 
             self._if_test(keyword, test, consequence, current_klass)
-            
+
         if node.else_:
             keyword = "else"
             test = None
             consequence = node.else_
 
             self._if_test(keyword, test, consequence, current_klass)
-            
-        
+
+
     def _if_test(self, keyword, test, consequence, current_klass):
-    
+
         if test:
             expr = self.expr(test, current_klass)
-    
+
             print >>self.output, "    " + keyword + " (" + expr + ") {"
         else:
             print >>self.output, "    " + keyword + " {"
@@ -543,11 +543,11 @@ class Translator:
         return "!(" + expr + ")"
 
     def _or(self, node, current_klass):
-        expr = " || ".join([self.expr(child, current_klass) for child in node.nodes])
+        expr = "("+") || (".join([self.expr(child, current_klass) for child in node.nodes])+")"
         return expr
 
     def _and(self, node, current_klass):
-        expr = " && ".join([self.expr(child, current_klass) for child in node.nodes])
+        expr = "("+") && (".join([self.expr(child, current_klass) for child in node.nodes])+")"
         return expr
 
     def _for(self, node, current_klass):
@@ -582,10 +582,10 @@ class Translator:
             list_expr = self._callfunc(node.list, current_klass)
         else:
             raise TranslationError("unsupported type (in _for)", node.list)
-        
+
         lhs = "var " + assign_name
         iterator_name = "__" + assign_name
-        
+
         print >>self.output, """
         var %(iterator_name)s = %(list_expr)s.__iter__();
         try {
@@ -647,13 +647,13 @@ class Translator:
         return self.expr(node.left, current_klass) + " % " + self.expr(node.right, current_klass)
 
     def _invert(self, node, current_klass):
-        return "~" + self.expr(node.expr, current_klass)
+        return "("+"~" + self.expr(node.expr, current_klass)+")"
 
     def _bitand(self, node, current_klass):
-        return " & ".join([self.expr(child, current_klass) for child in node.nodes])
+        return "("+" & ".join([self.expr(child, current_klass) for child in node.nodes])+")"
 
     def _bitor(self, node, current_klass):
-        return " | ".join([self.expr(child, current_klass) for child in node.nodes])
+        return "("+" | ".join([self.expr(child, current_klass) for child in node.nodes])+")"
 
     def _subscript(self, node, current_klass):
         if node.flags == "OP_APPLY":
@@ -768,7 +768,7 @@ class PlatformParser:
 
     def setPlatform(self, platform):
         self.platform = platform
-        
+
     def parseModule(self, module_name, file_name):
         if self.parse_cache.has_key(file_name):
             mod = self.parse_cache[file_name]
@@ -776,7 +776,7 @@ class PlatformParser:
             print "Importing " + module_name
             mod = compiler.parseFile(file_name)
             self.parse_cache[file_name] = mod
-        
+
         platform_file_name = self.generatePlatformFilename(file_name)
         if self.platform and os.path.isfile(platform_file_name):
             mod = copy.deepcopy(mod)
@@ -784,11 +784,11 @@ class PlatformParser:
             self.merge(mod, mod_override)
 
         return mod
-        
+
     def generatePlatformFilename(self, file_name):
         (module_name, extension) = os.path.splitext(os.path.basename(file_name))
         platform_file_name = module_name + self.platform + extension
-        
+
         return os.path.join(os.path.dirname(file_name), self.platform_dir, platform_file_name)
 
     def merge(self, tree1, tree2):
@@ -799,7 +799,7 @@ class PlatformParser:
                 self.replaceClassMethods(tree1, child.name, child)
 
         return tree1
-            
+
     def replaceFunction(self, tree, function_name, function_node):
         # find function to replace
         for child in tree.node:
@@ -815,10 +815,10 @@ class PlatformParser:
             if isinstance(child, ast.Class) and child.name == class_name:
                 old_class_node = child
                 break
-        
+
         if not old_class_node:
             raise TranslationError("class not found: " + class_name, class_node)
-        
+
         # replace methods
         for function_node in class_node.code:
             if isinstance(function_node, ast.Function):
@@ -846,7 +846,7 @@ class AppTranslator:
 
         self.library_modules = []
         self.library_dirs = library_dirs
-        
+
         if not parser:
             self.parser = PlatformParser()
         else:
@@ -855,37 +855,37 @@ class AppTranslator:
     def findFile(self, file_name):
         if os.path.isfile(file_name):
             return file_name
-        
+
         for library_dir in self.library_dirs:
             full_file_name = os.path.join(os.path.dirname(__file__), library_dir, file_name)
             if os.path.isfile(full_file_name):
                 return full_file_name
-        
+
         raise Exception("file not found: " + file_name)
-    
+
     def translate(self, module_name, is_app=True):
 
         if module_name not in self.library_modules:
             self.library_modules.append(module_name)
-        
+
         file_name = self.findFile(module_name + self.extension)
-        
+
         if is_app:
             module_name_translated = ""
         else:
             module_name_translated = module_name
-        
+
         output = cStringIO.StringIO()
-        
+
         mod = self.parser.parseModule(module_name, file_name)
         t = Translator(module_name_translated, mod, output)
         module_str = output.getvalue()
-        
+
         imported_modules_str = ""
         for module in t.imported_modules:
             if module not in self.library_modules:
                 imported_modules_str += self.translate(module, False)
-        
+
         return imported_modules_str + module_str
 
     def translateLibraries(self, library_modules=[]):
@@ -894,7 +894,7 @@ class AppTranslator:
         imported_modules_str = ""
         for library in self.library_modules:
             imported_modules_str += self.translate(library, False)
-        
+
         return imported_modules_str
 
 
