@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009 Net-ng.
+# Copyright (c) 2008, 2009, 2010 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -11,14 +11,14 @@ import threading, time
 
 import memcache
 
-from nagare.sessions import common
+from nagare.sessions import ExpirationError, common
 
 KEY_PREFIX = 'nagare_'
 
 class Lock(object):
     def __init__(self, connection, lock_id, ttl, poll_time, max_wait_time):
         """Distributed lock in memcache
-        
+
         In:
           - ``connection`` -- connection object to the memcache server
           - ``lock_id`` -- unique lock identifier
@@ -31,7 +31,7 @@ class Lock(object):
         self.ttl = ttl
         self.poll_time = poll_time
         self.max_wait_time = max_wait_time
-        
+
     def acquire(self):
         """Acquire the lock
         """
@@ -46,16 +46,30 @@ class Lock(object):
 
 
 class Sessions(common.Sessions):
-    """Sessions manager for sessions kept in an external memcache server
+    """Sessions manager for sessions kept in an external memcached server
     """
+    spec = dict(
+                host='string(default="127.0.0.1")',
+                port='integer(default=11211)',
+                ttl='integer(default=0)',
+                lock_ttl='float(default=0.)',
+                lock_poll_time='float(default=0.1)',
+                lock_max_wait_time='float(default=5.)',
+                min_compress_len='integer(default=0)',
+                reset='boolean(default=True)',
+                debug='boolean(default=False)',
+               )
+    spec.update(common.Sessions.spec)
+
     def __init__(
                  self,
                  host='127.0.0.1', port=11211,
                  ttl=0,
                  lock_ttl=0, lock_poll_time=0.1, lock_max_wait_time=5,
                  min_compress_len=0,
-                 reset=True,
-                 debug=True
+                 reset=False,
+                 debug=True,
+                 **kw
                 ):
         """Initialization
 
@@ -68,126 +82,174 @@ class Sessions(common.Sessions):
           - ``lock_max_wait_time`` -- maximum time to wait to acquire the lock, in seconds
           - ``min_compress_len`` -- data longer than this value are sent compressed
           - ``reset`` -- do a reset of all the sessions on startup ?
-          - ``debug`` -- display the memcache requests / responses 
-        """        
-        self.host = '%s:%d' % (host, port)
+          - ``debug`` -- display the memcache requests / responses
+        """
+        super(Sessions, self).__init__(**kw)
+
+        self.host = ['%s:%d' % (host, port)]
         self.ttl = ttl
         self.lock_ttl = lock_ttl
         self.lock_poll_time = lock_poll_time
         self.lock_max_wait_time = lock_max_wait_time
         self.min_compress_len = min_compress_len
         self.debug = debug
+
         self.memcached = threading.local()
 
+        if reset:
+            self.flush_all()
+
+    def set_config(self, filename, conf, error):
+        """Read the configuration parameters
+
+        In:
+          - ``filename`` -- the path to the configuration file
+          - ``conf`` -- the ``ConfigObj`` object, created from the configuration file
+          - ``error`` -- the function to call in case of configuration errors
+        """
+        # Let's the super class validate the configuration file
+        conf = super(Sessions, self).set_config(filename, conf, error)
+
+        self.host = ['%s:%d' % (conf['host'], conf['port'])]
+
+        for arg_name in (
+                            'ttl', 'lock_ttl', 'lock_poll_time', 'lock_max_wait_time',
+                            'min_compress_len', 'debug'
+                          ):
+            setattr(self, arg_name, conf[arg_name])
+
+        if conf['reset']:
+            self.flush_all()
+
     def _get_connection(self):
         """Get the connection to the memcache server
-        
+
         Return:
           - the connection
         """
         # The connection objects are thread local
         connection = self.memcached.__dict__.get('connection')
-        
+
         if connection is None:
-            connection = memcache.Client([self.host], debug=self.debug)
+            connection = memcache.Client(self.host, debug=self.debug)
             self.memcached.connection = connection
-            
+
         return connection
-            
+
+    def flush_all(self):
+        """Delete all the contents in the memcached server
+        """
+        memcached = memcache.Client(self.host, debug=self.debug)
+        memcached.flush_all()
+
     def _create(self, session_id, secure_id):
         """Create a new session
-        
+
         In:
           - ``session_id`` -- id of the session
           - ``secure_id`` -- the secure number associated to the session
-          
+
         Return:
-          - tuple (session_id, cont_id, new_cont_id, lock, secure_id)
+          - the tuple:
+            - id of this state,
+            - session lock
         """
         connection = self._get_connection()
         lock = Lock(connection, session_id, self.lock_ttl, self.lock_poll_time, self.lock_max_wait_time)
         lock.acquire()
-        
+
         connection.set_multi({
             '_sess' : (secure_id, None),
-            '_cont' : '0',
+            '_state' : '0',
             '00000' : {}
         }, self.ttl, KEY_PREFIX+session_id, self.min_compress_len)
-        
-        return (session_id, 0, 0, lock, secure_id)
-    
-    def __get(self, session_id, cont_id):
-        """Return the raw data associated to a session
-        
+
+        return (0, lock)
+
+    def _get(self, session_id, state_id, use_same_state):
+        """Retrieve the state
+
         In:
-          - ``session_id`` -- id of the session
-          - ``cont_id`` -- id of the continuation
-          
+          - ``session_id`` -- session id of this state
+          - ``state_id`` -- id of this state
+          - ``use_same_state`` -- is a copy of this state to create ?
+
         Return:
-            - tuple (session_id, cont_id, last_cont_id, lock, secure_id, externals, data)        
-        """        
+          - the tuple:
+            - id of this state,
+            - session lock,
+            - secure number associated to the session,
+            - data keept into the session
+            - data keept into the state
+        """
         connection = self._get_connection()
         lock = Lock(connection, session_id, self.lock_ttl, self.lock_poll_time, self.lock_max_wait_time)
         lock.acquire()
 
-        id = cont_id.zfill(5)
-        session = connection.get_multi(('_sess', '_cont', id), KEY_PREFIX+session_id)
+        state_id = state_id.zfill(5)
+        session = connection.get_multi(('_sess', '_state', state_id), KEY_PREFIX+session_id)
 
         if len(session) != 3:
-            raise common.ExpirationError()
+            raise ExpirationError()
 
-        last_cont_id = int(session['_cont'])
-        (secure_id, externals) = session['_sess']
-        data = session[id]
+        (secure_id, session_data) = session['_sess']
+        last_state_id = int(session['_state'])
+        state_data = session[state_id]
+
+        if not use_same_state:
+            state_id = last_state_id
+
+        return (int(state_id), lock, secure_id, session_data, state_data)
+
+    def _set(self, session_id, state_id, secure_id, use_same_state, session_data, state_data):
+        """Store the state
 
-        return (session_id, int(cont_id), last_cont_id, lock, secure_id, externals, data)
-    
-    def __set(self, session_id, cont_id, secure_id, inc_cont_id, externals, data):
-        """Memorize the session data
-        
         In:
-          - ``session_id`` -- id of the current session
-          - ``cont_id`` -- id of the current continuation
-          - ``secure_id`` -- the secure number associated to the session           
-          - ``inc_cont_id`` -- is the continuation id to increment ? 
-          - ``externals`` -- pickle of shared objects across the continuations                    
-          - ``data`` -- pickle of the objects in the continuation
+          - ``session_id`` -- session id of this state
+          - ``state_id`` -- id of this state
+          - ``secure_id`` -- the secure number associated to the session
+          - ``use_same_state`` -- is this state to be stored in the previous snapshot ?
+          - ``session_data`` -- data keept into the session
+          - ``state_data`` -- data keept into the state
         """
-        if inc_cont_id:
-            self._get_connection().incr(KEY_PREFIX+session_id+'_cont')
-        
+        if not use_same_state:
+            self._get_connection().incr(KEY_PREFIX+session_id+'_state')
+
         self._get_connection().set_multi({
-            '_sess' : (secure_id, externals),
-            '%05d' % cont_id : data
+            '_sess' : (secure_id, session_data),
+            '%05d' % state_id : state_data
         }, self.ttl, KEY_PREFIX+session_id, self.min_compress_len)
-        
-        
+
+
     def _delete(self, session_id):
         """Delete the session
-        
+
         In:
           - ``session_id`` -- id of the session to delete
         """
         self._get_connection().delete(KEY_PREFIX+session_id)
- 
- 
-class SessionsFactory(common.SessionsFactory):
-    spec = dict(
-                host='string(default="127.0.0.1")',
-                port='integer(default=11211)',
-                ttl='integer(default=0)',
-                lock_ttl='float(default=0.)',
-                lock_poll_time='float(default=0.1)',
-                lock_max_wait_time='float(default=5.)',
-                min_compress_len='integer(default=0)',
-                reset='boolean(default=True)',
-                debug='boolean(default=False)',
-               )
-    sessions = Sessions
 
-    def __init__(self, filename, conf, error):
-        super(SessionsFactory, self).__init__(filename, conf, error)
+    def serialize(self, data):
+        """Pickle an objects graph
 
-        if self.conf['reset']:
-            memcached = memcache.Client(['%s:%d' % (self.conf['host'], self.conf['port'])], debug=self.conf['debug'])
-            memcached.flush_all()
+        In:
+          - ``data`` -- the objects graphs
+
+        Return:
+          - the tuple:
+            - data to keep into the session
+            - data to keep into the state
+        """
+        return self.pickle(data)
+
+    def deserialize(self, session_data, state_data):
+        """Unpickle an objects graph
+
+        In:
+          - ``session_data`` -- data from the session
+          - ``state_data`` -- data from the state
+
+        Out:
+          - the objects graph
+        """
+        return self.unpickle(session_data, state_data)
