@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009 Net-ng.
+# Copyright (c) 2008, 2009, 2010 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -21,20 +21,19 @@ from nagare.namespaces import xml
 from nagare import presentation
 from nagare import component
 from nagare import wsgi
-from nagare.sessions import threaded_sessions
+from nagare.sessions.memory_sessions import SessionsWithPickledStates
 
 from exceptions import Exception
 
 def create_FixtureApp(app):
     app = wsgi.create_WSGIApp(app)
-    app.start(threaded_sessions.SessionsFactory('', {}, lambda x: None)())
+    app.set_sessions_manager(SessionsWithPickledStates())
+    app.start()
 
     return TestApp(app)
 
-dabasename = 'test_db.sqlite'
-
 __metadata__ = MetaData()
-__metadata__.bind = "sqlite:///%s" % dabasename
+__metadata__.bind = "sqlite:///:memory:"
 __metadata__.bind.echo = False
 
 # Setup / Teardown functions
@@ -42,11 +41,6 @@ def setup():
     setup_all()
 
 
-def teardown():
-    #cleanup_all(drop_tables=True)
-    os.remove(dabasename)
-
-
 def setup_func():
     create_all()
 
@@ -202,7 +196,7 @@ def render(self, h, comp, *args):
 
 @with_setup(setup_func_2, teardown_func)
 def test5():
-    """ Test database - Get element in database """
+    """ database - Get element in database """
     myApp = My_database_app
     app = create_FixtureApp(myApp)
     res = app.get('/')
@@ -211,7 +205,7 @@ def test5():
 
 @with_setup(setup_func_2, teardown_func)
 def test6():
-    """ Test database - Add element in database """
+    """ database - Add element in database """
     myApp = My_database_app
     app = create_FixtureApp(myApp)
     res = app.get('/')
@@ -225,7 +219,7 @@ def test6():
 
 @with_setup(setup_func_2, teardown_func)
 def test7():
-    """ Test database - Add 2 elements in database """
+    """ database - Add 2 elements in database """
     myApp = My_database_app
     app = create_FixtureApp(myApp)
     res = app.get('/')
@@ -244,7 +238,7 @@ def test7():
 
 @with_setup(setup_func_2, teardown_func)
 def test8():
-    """ Test database - Add 2 elements in database and test rollback on Exception """
+    """ database - Add 2 elements in database and test rollback on Exception """
     myApp = My_database_app
     app = create_FixtureApp(myApp)
     res = app.get('/')
@@ -268,5 +262,3 @@ def test8():
     assert u"Mary Ingalls" in res
     res = res.click(linkid="get_name")
     assert u"Charles Ingalls" in res
-
-
