@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, 2009 Net-ng.
+# Copyright (c) 2008, 2009, 2010 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -16,15 +16,17 @@ expired session id, it creates a new root component and a new session.
 """
 from __future__ import with_statement
 
+import sys
+
 import webob
 from webob import exc, acceptparse
 
-from nagare import component, presentation, serializer, database, top, security
+from nagare import component, presentation, serializer, database, top, security, log, comet
 from nagare.security import dummy_manager
 from nagare.callbacks import Callbacks, CallbackLookupError
 from nagare.namespaces import xhtml
 
-from nagare.sessions.common import ExpirationError
+from nagare.sessions import ExpirationError
 
 # ---------------------------------------------------------------------------
 
@@ -37,48 +39,93 @@ class MIMEAcceptWithoutWildcards(acceptparse.Accept):
 # ---------------------------------------------------------------------------
 
 class WSGIApp(object):
-    def __init__(self, root_factory, metadatas=None):
+    def __init__(self, root_factory):
         """Initialization
 
         In:
           - ``root_factory`` -- function to create the application root component
-          - ``metadatas`` -- the SQLAlchemy metadata objects
+          - ``databases`` -- the SQLAlchemy metadata objects and the database engines settings
         """
         self.root_factory = root_factory
-        self.metadatas = metadatas or []
 
-        self.static_url = ''
         self.static_path = ''
+        self.static_url = ''
+        self.databases = []
+        self.name = ''
+        self.project_name = ''
         self.redirect_after_post = False
         self.always_html = True
+        self.sessions = None
+        self.last_exception = None
 
         self.security = dummy_manager.Manager()
 
-    def set_config(self, config_filename, config, error, static_path, static_url, publisher):
+    def set_config(self, config_filename, config, error):
         """Read the configuration parameters
 
         In:
           - ``config_filename`` -- the path to the configuration file
           - ``config`` -- the ``ConfigObj`` object, created from the configuration file
           - ``error`` -- the function to call in case of configuration errors
-          - ``static_path`` -- the directory where the static contents of the application
-            are located
-          - ``static_url`` -- the url of the static contents of the application
-          - ``publisher`` -- the publisher of the application
         """
-        self.static_path = static_path
-        self.static_url = static_url
-
+        self.name = config['application']['name']
         self.redirect_after_post = config['application']['redirect_after_post']
         self.always_html = config['application']['always_html']
 
-    def start(self, sessions):
-        """Call after each process start
+    def set_static_path(self, static_path):
+        """Register the directory of the static contents
 
         In:
-          - ``sessions`` -- the sessions manager for this application
+          - ``static_path`` -- the directory where the static contents of the application
+            are located
         """
-        self.sessions = sessions
+        self.static_path = static_path
+
+    def set_static_url(self, static_url):
+        """Register the url of the static contents
+
+        In:
+          - ``static_url`` -- the url of the static contents of the application
+        """
+        self.static_url = static_url
+
+    def set_publisher(self, publisher):
+        """Register the publisher
+
+        In:
+          - ``publisher`` -- the publisher of the application
+        """
+        pass
+
+    def set_sessions_manager(self, sessions_manager):
+        """Register the sessions manager
+
+        In:
+          - ``sessions_manager`` -- the sessions manager
+        """
+        self.sessions = sessions_manager
+
+    def set_databases(self, databases):
+        """Register the databases properties
+
+        In:
+          - ``databases`` -- the SQLAlchemy metadata objects and the database engines settings
+        """
+        self.databases = databases
+
+    def start(self):
+        """Call after each process start
+        """
+        for database_settings in self.databases:
+            database.set_metadata(*database_settings)
+
+    def set_project(self, name):
+        """The application distribution name
+
+        In:
+          - ``project_name`` -- name of the distutils distribution where the app is located
+        """
+        self.project_name = name
 
     # -----------------------------------------------------------------------
 
@@ -131,23 +178,23 @@ class WSGIApp(object):
           - a tree
         """
         return output
-    
+
     def on_callback_lookuperror(self, request, response, async):
-        """
+        """A callback was not found
+
         In:
           - ``request`` -- the web request object
           - ``response`` -- the web response object
           - ``async`` -- is an XHR request ?
-          
         """
         if not async:
             raise
 
         # As the XHR requests use the same continuation, a callback
         # can be not found (i.e deleted by a previous XHR)
-        # In this case, do nothing                     
+        # In this case, do nothing
         return lambda h: ''
-    
+
     def on_after_post(self, request, response, ids):
         """Generate a redirection after a POST
 
@@ -161,15 +208,27 @@ class WSGIApp(object):
         """
         return exc.HTTPSeeOther(location=request.path_url + '?' + '&'.join(ids))
 
+    def on_exception(self, request, response):
+        """Method called when an exception occurs
+
+        In:
+          - ``request`` -- the web request object
+          - ``response`` -- the web response object
+
+        Return:
+          - a ``webob`` response object
+        """
+        raise
+
     # -----------------------------------------------------------------------
 
-    def create_root(self):
+    def create_root(self, *args, **kw):
         """Create the application root component
 
         Return:
           - the root component
         """
-        return self.root_factory()
+        return self.root_factory(*args, **kw)
 
     def create_renderer(self, async, session, request, response, callbacks):
         """Create the initial renderer (the root of all the used renderers)
@@ -203,20 +262,21 @@ class WSGIApp(object):
         security.set_user(self.security.create_user(request, response)) # Create the User object
 
     # Processing phase
-    def _phase1(self, params, callbacks):
+    def _phase1(self, request, response, callbacks):
         """Phase 1 of the request processing:
 
           - The callbacks are processed
           - The objects graph can be modified
 
         In:
-          - ``params`` -- parameters received into the request
+          - ``request`` -- the web request object
+          - ``response`` -- the web response object
           - ``callbacks`` -- the registered callbacks
 
         Return:
           - function to render the objects graph or ``None``
         """
-        return callbacks.process_response(params)
+        return callbacks.process_response(request, response)
 
     # Rendering phase
     def _phase2(self, request, response, output, is_xhr):
@@ -228,8 +288,8 @@ class WSGIApp(object):
           - No modification of the objects is allowed
 
         In:
-          - ``session`` -- the session
           - ``request`` -- the web request object
+          - ``response`` -- the web response object
           - ``output`` -- the rendered tree
           - ``is_xhr`` -- is the request a XHR request ?
 
@@ -254,13 +314,22 @@ class WSGIApp(object):
 
         request = webob.Request(environ, charset='utf-8')
         response = webob.Response(headerlist=[])
-        
+
         accept = MIMEAcceptWithoutWildcards('Accept', 'text/html' if self.always_html else str(request.accept))
         response.xhtml_output = accept.best_match(('text/html', 'application/xhtml+xml')) == 'application/xhtml+xml'
 
+        channel_id = request.params.get('_channel')
+        nb = request.params.get('_nb')
+        if channel_id and nb:
+            comet.channels.connect(environ['wsgi.multiprocess'], channel_id, int(nb), response)
+            return response(environ, start_response)
+
         xhr_request = request.is_xhr or ('_a' in request.params)
 
         session = None
+        self.last_exception = None
+
+        log.set_logger('nagare.application.'+self.name) # Set the dedicated application logger
 
         # Create a database transaction for each request
         with database.session.begin():
@@ -273,7 +342,7 @@ class WSGIApp(object):
                     self.on_incomplete_url(request, response)
 
                 try:
-                    session = self.sessions.get(request, response)
+                    session = self.sessions.get(request, response, xhr_request)
                 except ExpirationError:
                     self.on_session_expired(request, response)
 
@@ -298,13 +367,16 @@ class WSGIApp(object):
                     presentation.init(root, tuple([u.decode('utf-8') for u in url.split('/')]), None, request.method, request)
 
                 try:
-                    render = self._phase1(request.params, callbacks)
+                    render = self._phase1(request, response, callbacks)
                 except CallbackLookupError:
                     render = self.on_callback_lookuperror(request, response, xhr_request)
             except exc.HTTPException, response:
                 # When a ``webob.exc`` object is raised during phase 1, skip the
                 # phase 2 and use it as the response object
                 pass
+            except Exception:
+                self.last_exception = (request,  sys.exc_info())
+                response = self.on_exception(request, response)
             else:
                 # Phase 2
                 # -------
@@ -316,10 +388,10 @@ class WSGIApp(object):
 
                 try:
                     if (request.method == 'POST') and not xhr_request and self.redirect_after_post:
-                        store_new_cont = False
+                        use_same_state = True
                         response = self.on_after_post(request, response, session.sessionid_in_url(request, response))
                     else:
-                        store_new_cont = not xhr_request
+                        use_same_state = xhr_request
 
                         # Create a new renderer
                         renderer = self.create_renderer(xhr_request, session, request, response, callbacks)
@@ -339,14 +411,16 @@ class WSGIApp(object):
                             callbacks.clear_not_used(renderer._rendered)
 
                     # Store the session
-                    session.data = (root, callbacks)
-                    self.sessions.set(session, store_new_cont)
+                    session.set(use_same_state, (root, callbacks))
 
-                    security.get_manager().end_rendering(request, response, self.sessions, session)
+                    security.get_manager().end_rendering(request, response, session)
                 except exc.HTTPException, response:
                     # When a ``webob.exc`` object is raised during phase 2, stop immediatly
                     # use it as the response object
                     pass
+                except Exception:
+                    self.last_exception = (request,  sys.exc_info())
+                    response = self.on_exception(request, response)
             finally:
                 if session:
                     session.lock.release()
@@ -355,7 +429,7 @@ class WSGIApp(object):
 
 # ---------------------------------------------------------------------------
 
-def create_WSGIApp(app, metadata=None, with_component=True):
+def create_WSGIApp(app, with_component=True):
     """Helper function to create a WSGIApp
 
     If ``app`` is not a ``WSGIApp``, it's wrap into a ``WSGIApp``. And, if
@@ -375,6 +449,6 @@ def create_WSGIApp(app, metadata=None, with_component=True):
                 return o
             app = lambda app=app: wrap_in_component(app)
 
-        app = WSGIApp(app, metadata)
+        app = WSGIApp(app)
 
     return app
